@@ -83,7 +83,6 @@ struct jr3_pci_poll_delay {
 struct jr3_pci_dev_private {
 	struct timer_list timer;
 	struct comedi_device *dev;
-	bool timer_enable;
 };
 
 union jr3_pci_single_range {
@@ -608,11 +607,10 @@ static void jr3_pci_poll_dev(struct timer_list *t)
 				delay = sub_delay.max;
 		}
 	}
-	if (devpriv->timer_enable) {
-		devpriv->timer.expires = jiffies + msecs_to_jiffies(delay);
-		add_timer(&devpriv->timer);
-	}
 	spin_unlock_irqrestore(&dev->spinlock, flags);
+
+	devpriv->timer.expires = jiffies + msecs_to_jiffies(delay);
+	add_timer(&devpriv->timer);
 }
 
 static struct jr3_pci_subdev_private *
@@ -761,7 +759,6 @@ static int jr3_pci_auto_attach(struct comedi_device *dev,
 	devpriv->dev = dev;
 	timer_setup(&devpriv->timer, jr3_pci_poll_dev, 0);
 	devpriv->timer.expires = jiffies + msecs_to_jiffies(1000);
-	devpriv->timer_enable = true;
 	add_timer(&devpriv->timer);
 
 	return 0;
@@ -771,12 +768,8 @@ static void jr3_pci_detach(struct comedi_device *dev)
 {
 	struct jr3_pci_dev_private *devpriv = dev->private;
 
-	if (devpriv) {
-		spin_lock_bh(&dev->spinlock);
-		devpriv->timer_enable = false;
-		spin_unlock_bh(&dev->spinlock);
+	if (devpriv)
 		del_timer_sync(&devpriv->timer);
-	}
 
 	comedi_pci_detach(dev);
 }
@@ -812,7 +805,7 @@ static struct pci_driver jr3_pci_pci_driver = {
 };
 module_comedi_pci_driver(jr3_pci_driver, jr3_pci_pci_driver);
 
-MODULE_AUTHOR("Comedi http://www.comedi.org");
+MODULE_AUTHOR("Comedi https://www.comedi.org");
 MODULE_DESCRIPTION("Comedi driver for JR3/PCI force sensor board");
 MODULE_LICENSE("GPL");
 /*(DEBLOBBED)*/
